__doc__='''

Zieve class for sorting objects based on runtime attributes...

Steve Spicklemire
Silicon Prairie Ventures Inc.
steve@spvi.com

'''

__doc__ = "Zieve Flexible Sorter"
__version__ = "0.1.0"

from Globals import HTMLFile, MessageDialog
import Globals
from Acquisition import Implicit
from Persistence import Persistent
from OFS.SimpleItem import SimpleItem

import compClass
import string

DEFMAXKEYS=3  # default number of keys to keep around..

manage_addZieveForm=HTMLFile('addZieve',globals())

def manage_addZieve(self, id, title, useCookies=0, REQUEST=None):
    """Add a Zieve object
    """
    c=Zieve(id,title)
    self._setObject(id,c)

    cp = string.split(self.absolute_url(),'//')[1]
    cp = string.split(cp,'/')[1:]

    c.cookiePath = '/' + string.join(cp,'/')
    c.useCookies = useCookies
    
    if REQUEST is not None:
        return self.manage_main(self,REQUEST)

class Zieve(SimpleItem, Persistent, Implicit):

    """Zieve object"""

    meta_type = "Zieve"
    icon='misc_/Zieve/zieve.gif'
    cookiePath = '/'
    typesToSort = None
    defaultOrder = None
    useCookies = 0
    
    manage_options=(
        {'label': 'Properties', 'action': 'manage_edit_form'},
        {'label': 'Security', 'action': 'manage_access' },
        {'label': 'View/Test', 'action':'exampleTable'},
        )

    manage_edit_form = HTMLFile('manage_edit_form',globals())
    index_html = HTMLFile('zieveView', globals())
    exampleTable = HTMLFile('exampleTable', globals())

    def __init__(self,id,title=''):
        self.id=id
        self.title=title
        self.typesToSort = []
        self.defaultOrder = []

    def getParent(self):
        """ Get my parent object or self..... make sure it has 'objectValues'"""

        result = None
        
        if hasattr(self, 'aq_parent'):
            theParent = getattr(self, 'aq_parent')
            if hasattr(theParent, 'objectValues'):
                result = theParent
        else:
            if hasattr(self, objectValues):
                result = self
                
        return result

    def reload_example(self, REQUEST=None):
        """ Just do it"""

        Zieve.index_html = HTMLFile('exampleTable', globals())

        return self.manage_edit_form(self, REQUEST, manage_tabs_message="Examples reloaded..")

    
    def manage_edit(self, title='', cookiePath=None, typesToSort=None, defaultOrder=None, useCookies=0, REQUEST=None):
        """ edit the catalog """

        message = "Object changed"

        self.title = title
	self.cookiePath = cookiePath
        self.useCookies = useCookies

	# zap any leading '[' or trailing ']' ...
	
	while len(typesToSort) and (typesToSort[0] in [' ', '[', ']', "'" ]):
	    typesToSort = typesToSort[1:]
	    
	while len(typesToSort) and (typesToSort[-1] in [' ', '[', ']',"'" ]):
	    typesToSort = typesToSort[:-1]

	if len(typesToSort):
	    typesToSort = string.replace(typesToSort,"'","")  # zap any "'"s.
	    newTypes = []

	    typesToSort = string.split(typesToSort,',')
            
	    for theType in typesToSort:
		while len(theType) and (theType[0] in [' ', '[', ']', "'" ]):
		    theType = theType[1:]

		while len(theType) and (theType[-1] in [' ', '[', ']', "'" ]):
		    theType = theType[:-1]

		if len(theType):
		    newTypes.append(theType)

            self.typesToSort = newTypes
        else:
            self.typesToSort = ''

        self.defaultOrder = defaultOrder

        return self.manage_edit_form(self, REQUEST,
				     manage_tabs_message=message)

    def sortObjects(self, explicitOrder=None, theClass=None, theObjects=None, sortOrder=None, zieveOptions=None, REQUEST=None, RESPONSE=None):
        """
        sort a list of objects using the zieveKeys in the REQUEST, or the defaultKeys otherwise...
        """

        if REQUEST:
            explicitOrder = REQUEST.get('explicitOrder',explicitOrder)
            theClass = REQUEST.get('theClass',theClass)
            theObjects = REQUEST.get('theObjects', theObjects)
            zieveOptions = REQUEST.get('sortOrder',sortOrder)  # goofiness due to deprecated 'zieveOptions' 
            if zieveOptions is None:
                zieveOptions = REQUEST.get('zieveOptions',zieveOptions)
        else:
            if sortOrder is not None:
                zieveOptions = sortOrder
            
        zieveKeys = self.parseZieveOptions(zieveOptions)

        if not zieveKeys:
            zieveKeys = explicitOrder
            if not zieveKeys:
                zieveKeys = self.defaultOrder
                
            if RESPONSE and self.useCookies:
                RESPONSE.setCookie('zieveOptions', zieveKeys, path=self.cookiePath)

        if theObjects is not None:
            if type(theObjects) == type(()):
                theObjects = list(theObjects)
                
            theObjects.sort(compClass.compClass(zieveKeys, raiseExceptions = 0, theClass = theClass))
        else:
            if self.typesToSort:
                myParent = self.getParent()
                if myParent:
                    theObjects = myParent.objectValues(self.typesToSort)
                    if theObjects:
                        theObjects.sort(compClass.compClass(zieveKeys, raiseExceptions = 0, theClass = theClass))

        return theObjects

    def parseZieveOptions(self, zieveOptions, maxKeys=DEFMAXKEYS, newKey=None):
        """
          zieveOptions will be a list of strings like so:

            'title:char',
            'positionOption'
            'poid:int'
            'foo:None'

            In general that's  'attributename' [ :'sort type' ]
            
            which means sort by certain attributes interpreted as a certain
            type with in the key-order given. If a new key is added, put
            that in first..

        """
        #

        result = {}
        badchars = "'\"[] "

        if zieveOptions is None:
            zieveOptions = []

        if type(zieveOptions) == type(''):  # dang it's a string... convert it...
            theList = string.split(zieveOptions,',')
            zieveOptions = []
            for item in theList:
                zieveOptions.append(self.stripBad(item, badchars))

        if newKey is not None:
            newKey = self.stripBad(newKey, badchars)            

        if newKey:
            if zieveOptions:
                if (newKey in zieveOptions):
                    # move it to front of list.... get rid of the old value...
                    theIndex = zieveOptions.index(newKey)
                    del zieveOptions[theIndex]
                    
                zieveOptions.insert(0, newKey)
            else:
                zieveOptions.insert(0, newKey)

        return zieveOptions[:maxKeys]  # maximum number of sort keys.....

    def setSortPrimary(self, REQUEST, RESPONSE, URL1=None, maxKeys=DEFMAXKEYS):
        """
        set the primary key for the sort....
        """

        if not self.useCookies:
            raise "Bad Request", "Not much point in setSortPrimary without cookies... "
        
        newKey = REQUEST.get('key','')
        zieveOptions = self.parseZieveOptions(REQUEST.get('zieveOptions',None), maxKeys, newKey)
        RESPONSE.setCookie('zieveOptions', zieveOptions, path=self.cookiePath)
        RESPONSE.redirect(REQUEST.get('HTTP_REFERER',''))
        
    def stripBad(self, item, badchars):

        while len(item) and item[0] in badchars:
            item = item[1:]
        while len(item) and item[-1] in badchars:
            item = item[:-1]
        return item

    def normalSortList(self, sortItem ):
        """
            given a sort item string... return the list:
            ['attr','type']

        """
        #
        theList = string.split(sortItem,':')

        if len(theList) == 1:
            theList = [theList[0],'']

        elif len(theList) >= 2:
            # assume that's attr:sortorder
            theList = theList[:2]

        return theList

        
Globals.default__class_init__(Zieve)


        
